             .title        "NROM Reader Plugin"


;06/10/00 
;Written by KH
;Version 1.0

;03/16/09
;BootGod
;Added support for 8K PRG ROM

;06/12/11
;Kinopio
;Changed how PRG-size is detected because the old
;way got Lunar Pool's (PAL) size wrong
             
             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
wr_ppu:      .equ 020ch

readAdr1:    .equ 00e0h
readAdr1_lo: .equ 00e0h
readAdr1_hi: .equ 00e1h
readAdr2:    .equ 00e2h
readAdr2_lo: .equ 00e2h
readAdr2_hi: .equ 00e3h
romsize:     .equ 00e6h

;plugin header defines
p_ver: = 3	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Nintendo NROM / ROM-only carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "Kevtris,BootGod,Kinopio"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 16 >> 4,32 >> 4
             .db 8 >> 3,0 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here

             
;check mirroring
             ldx #000h        ;0 = horz mirroring
             
             lda #020h
             jsr wr_ppu
             lda #055h
             sta 2007h
             lda #0AAh
             sta 2007h
             lda #024h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #055h
             bne horz_mir
             lda 2007h
             cmp #0AAh
             bne horz_mir
             lda #020h
             jsr wr_ppu
             lda #0AAh
             sta 2007h
             lda #024h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #0AAh
             bne horz_mir
             inx             ;1 = vert mirroring

horz_mir:    txa
             jsr send_byte   ;send mirroring byte

;determine if this is a 8,16K or 32K PRG game
             lda #080h
             sta romsize     ;set romsize to 32KB (128*256)
             
             lda #000h       
             sta readAdr1_lo
             lda #080h
             sta readAdr1_hi ;readAdr1 = 8000h             
             lda #000h
             sta readAdr2_lo
             lda #0C0h
             sta readAdr2_hi ;readAdr2 = C000h
             
             ldx #040h       ;64*256=16384
             ldy #000h
             
chk_loop1:   lda (readAdr1),y
             cmp (readAdr2),y;compare first and second 16KB PRG
             bne got_prg     ;if not equal then 32KB PRG
             iny
             bne chk_loop1   ;loop 256 times
             inc readAdr1_hi
             inc readAdr2_hi
             dex
             bne chk_loop1
             
             lsr romsize     ;set romsize to 16KB (64*256)
             
             lda #000h       
             sta readAdr1_lo
             lda #080h
             sta readAdr1_hi ;readAdr1 = 8000h             
             lda #000h
             sta readAdr2_lo
             lda #0A0h
             sta readAdr2_hi ;readAdr2 = A000h
             
             ldx #020h       ;32*256=8192
             ldy #000h
             
chk_loop2:   lda (readAdr1),y
             cmp (readAdr2),y;compare first and second 8KB PRG
             bne got_prg     ;if not equal then 16KB PRG
             iny
             bne chk_loop2   ;loop 256 times
             inc readAdr1_hi
             inc readAdr2_hi
             dex
             bne chk_loop2
             
             lsr romsize     ;set romsize to 8KB (32*256)
             
got_prg:     lda romsize
             jsr send_byte   ;send byte lo size
             lda #000h
             jsr send_byte   ;send byte hi size
             lda #001h       ;type is PRG
             jsr send_byte   ;send type

             ldy #000h       
             sty readAdr1_lo
             lda #080h
             sta readAdr1_hi ;readAdr1 = 8000h
             
dmpp_loop:   lda (readAdr1),y
             jsr send_byte
             iny
             bne dmpp_loop   ;send a byte 256 times
             jsr baton
             inc readAdr1_hi
             dec romsize
             bne dmpp_loop   ;repeat for romsize
             
;dump 8KB CHR
             lda #020h
             tax
             jsr send_byte   ;send byte lo size
             lda #000h
             jsr send_byte   ;send byte hi size
             lda #002h       ;type is CHR
             jsr send_byte   ;send type

;init chr
             lda #000h
             sta 02006h
             lda #000h
             sta 02006h     ;set PPU address to 0000h
             lda 02007h     ;read garbage byte
             ldy #0
             
dump_chr:    lda 02007h
             jsr send_byte
             iny
             bne dump_chr
             dex
             bne dump_chr   ;dump 8K of CHR
 
done:        lda #000h
             jsr send_byte  ;send end flag
             lda #000h
             jsr send_byte  ;send end flag
             lda #000h
             jsr send_byte  ;send end flag
             rts            ;done 

             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end
